function [normal_map albedo] = find_normal_albedo(mask_obj, imageList, L_list_1, L_list_2, L_mag)

imask_obj = rgb2gray(imread(mask_obj));

% find all object's pixels
[objectY objectX] = find(imask_obj > 127);

% allocate
imgHeight = length(imask_obj(:,1));
imgWidth = length(imask_obj(1,:));
imgListSize = length(imageList(:,1));
IL1 = zeros(imgListSize,3);
IL2 = zeros(imgListSize,3);
I = zeros(imgListSize,1);
I_square = zeros(imgListSize*2,1);
normal_map = zeros(imgHeight, imgWidth, 3);
images = zeros( length(imageList(:,1)), imgHeight, imgWidth );
albedo = zeros(imgHeight, imgWidth);

% normalize light intensity
disp('  - Normalizing light intensity...');
[min_mag min_ind] = min(L_mag);
coeff = min_mag./L_mag;
% preload images to  images
% disp('  - Loading images...');
for i=1:imgListSize
	
	img = rgb2gray(imread_pfm(imageList(i,:)));
	images(i,:,:) = img.*coeff(i);
end

disp('  - Estimating normals...');

for j=1:length(objectX(:))	% iterate all X Y of object
	
	for i=1:imgListSize
		I(i) = images(i, objectY(j), objectX(j));	% extract I
		% multiply both sides with I
		IL1(i,:) = I(i) .* L_list_1(i,:);
		IL2(i,:) = I(i) .* L_list_2(i,:);
		I(i) = I(i).^2;
	end
	
	% form the linear least square system I_square = IL*G
	IL = [IL1; IL2];
	I_square = [I; I];
	I_square(all(IL==0,2),:)=[];		% remove zero entires
	IL(all(IL==0,2),:)=[];		% remove zero entires
	% use SVD econ to solve for G
	[U,S,V] = svd(IL,'econ');
	s = diag(S);	% reshape S
	d = U'*I_square;
	G = V*(d./s);	% G
	k_d = norm(G);	% length of G is k_d
	normal = G/k_d;	%
	% save result
	normal_map(objectY(j), objectX(j), :) = normal;
	albedo(objectY(j), objectX(j)) = k_d;
end
normal_mapRGB = (normal_map + 1) ./ 2;	% shift&scale normals
imwrite(normal_mapRGB, 'normal_map.png');
figure;
imshow(normal_mapRGB);
pause(0.5);
% normalize albedo image
max_albedo = max(max(albedo));
albedo = albedo ./ max_albedo;
imwrite(albedo,'albedo.png');
% figure;
% imshow(albedo);
end